CREATE PROCEDURE "ImportInternetOrders" (IN aFileName VARCHAR, OUT Result VARCHAR)
BEGIN
  /*
      Use aFileName to call IMPORT, which pulls data into the database table "WebShopOrders".
      SELECT imported records, linking to Products for ProdCode
      Iterate these records, inserting SalesInvoices and SalesInvoiceItems.
      UPDATE the WebShopOrders" table, marking records as "Imported"
      Send an email informing Rachel and Josh that the new records have been imported.
  */
  DECLARE Crsr CURSOR FOR Stmt;
  DECLARE ItemCrsr CURSOR FOR ItemStmt;
  DECLARE SalesInvoiceID Integer;
  DECLARE WorkingOrderNumber VARCHAR;
  DECLARE OrderNumber VARCHAR;
  DECLARE RecipientName VARCHAR;
  DECLARE ShippingDetails VARCHAR;
  DECLARE Email VARCHAR;
  DECLARE CustomerNote VARCHAR;
  DECLARE WebKey VARCHAR;
  DECLARE TotalValue DECIMAL;
  DECLARE ShippingTotal DECIMAL;
  DECLARE DateWanted DATE;
  DECLARE ProductID INTEGER;
  DECLARE CaseSingle VARCHAR;
  DECLARE CaseSize FLOAT;
  DECLARE ItemTotal DECIMAL;
  DECLARE ItemQuantity FLOAT;
  DECLARE PhoneNumber VARCHAR;
  DECLARE SalesInvoiceIDs VARCHAR; --holds all newly added orders today
  DECLARE ProcessedOrderNumbers VARCHAR;
  DECLARE OverallTotalValue DECIMAL;
  DECLARE EmailRunnerResult VARCHAR; 
  DECLARE OrderCount INTEGER;                                               
--WorkingOrderNumber is used to hold the "current" order, so multi-line orders can be sequenced as we work through the data.   
SET ProcessedOrderNumbers = '';
SET SalesInvoiceIDs = '';
SET WorkingOrderNumber = '';  
SET OverallTotalValue = 0;
SET OrderCount = 0;
SET Result = 'Running Import Internet Orders Procedure' + #13;
EXECUTE IMMEDIATE
 'IMPORT TABLE WebShopOrders
  FROM "' + aFileName + '"
  IN STORE "Imports"
  FORMAT DELIMITED
  USE HEADERS';
SET Result = Result + 'File: ' + aFileName + ' imported into the "Imports" store.' + #13;    
EXECUTE IMMEDIATE
 ' RENAME FILE "' + aFileName + '" ' + 
 ' IN STORE Imports TO "' + aFileName + '-old"';                            
PREPARE Stmt FROM
  'SELECT 
    WS.order_number as OrderNumber,
    WS.shipping_first_name + '' '' + 
    WS.shipping_last_name as RecipientName, 
    COALESCE(WS.shipping_first_name + '' '', '''') + 
    COALESCE(WS.shipping_last_name + #13, '''') +
    COALESCE(WS.shipping_address_1 + #13, '''') + 
    COALESCE(WS.shipping_address_2 + #13, '''') + 
    COALESCE(WS.shipping_postcode + #13, '''') + 
    COALESCE(WS.shipping_city + #13, '''') + 
    COALESCE(WS.shipping_state + #13, '''') + 
    COALESCE(WS.shipping_country + #13, '''') +
    COALESCE(WS.billing_phone + #13, '''') 
       as ShippingDetails,
    WS.billing_email as Email,
    WS.billing_phone as Phone,
    WS.customer_note, 
    COALESCE(CAST(WS.shipping_total as FLOAT), 0) as shipping_total, 
    COALESCE(CAST(WS.order_subtotal as FLOAT), 0) as order_subtotal,    
    WS.Transaction_id as WebKey,
    CAST(WS.order_date as DATE DATE FORMAT ''DD/MM/YYYY'') as DateWanted,
    IF(POSITION(''-s'', WS.item_sku) > 0 THEN ''Single'' ELSE ''Case'') as CaseSingle,
    P.ID as ProductID,
    P.CountUnits as CaseSize,
    COALESCE(CAST(WS.item_quantity as FLOAT), 0) as item_quantity,
    COALESCE(CAST(WS.item_total as FLOAT), 0) as item_total
  FROM WebShopOrders WS
  LEFT JOIN Products P ON P.ProdCode = REPLACE(''-s'' WITH ''''  IN WS.item_sku)
  WHERE UPPER(WS.Status) = ''PROCESSING''
  ORDER BY OrderNumber, ID ';
OPEN Crsr;
SET Result = Result + COALESCE(CAST(ROWCOUNT(Crsr) AS VARCHAR) + ' WebOrder rows selected for processing' + #13, '');
FETCH FIRST FROM Crsr('OrderNumber', 'RecipientName', 'ShippingDetails', 'Email', 'Phone', 
                      'customer_note', 'shipping_total', 'order_subtotal', 'WebKey', 'DateWanted', 
                      'CaseSingle', 'ProductID', 'CaseSize', 'item_total', 'item_quantity')
            INTO OrderNumber, RecipientName, ShippingDetails, Email, PhoneNumber, CustomerNote, ShippingTotal,
                  TotalValue, WebKey, DateWanted, CaseSingle, ProductID, CaseSize, ItemTotal, ItemQuantity;                   
                  
WHILE NOT EOF(Crsr) DO
  IF NOT WorkingOrderNumber = OrderNumber THEN --iteration has brought us to a new order, so add a new sales invoice
    SET WorkingOrderNumber = OrderNumber;
    CALL InsertSalesInvoice(DateWanted,
                            540444,
                            WebKey, 
                            RecipientName + ' (' + OrderNumber + ')', 
                            Email + ' ' + PhoneNumber, 
                            CustomerNote, 
                            ShippingDetails,
                            10780,                                    
                            ShippingTotal,                                    
                            103,  
                            true, 
                            1052068,                                     
                            TotalValue,            
                            SalesInvoiceID);
    
    SET ProcessedOrderNumbers = ProcessedOrderNumbers + OrderNumber + ', ';
    SET SalesInvoiceIDs = SalesInvoiceIDs + CAST(SalesInvoiceID AS VARCHAR) + ', ';
    SET OverallTotalValue = OverallTotalValue + TotalValue;
    SET OrderCount = OrderCount + 1;
    END IF;
  --for every row, add a SaleInvoiceItem, using the existing SalesInvoiceID
  --if SINGLE then Qty = Qty / CaseSize
  IF CaseSingle = 'Case' THEN
    PREPARE ItemStmt FROM
      ' INSERT INTO SalesInvoiceItems
        (SalesInvoicesID, ProductsID,
         QtyOrdered, SalePrice, InternetPricePaid)
          VALUES
         (?, ?, ?, ?, ?) ';
    OPEN ItemCrsr USING SalesInvoiceID, ProductID, ItemQuantity, ItemTotal/ItemQuantity, ItemTotal; 
    CLOSE ItemCrsr;
    ELSE
      PREPARE ItemStmt FROM
        ' INSERT INTO SalesInvoiceItems
          (SalesInvoicesID, ProductsID,
           QtyOrdered, SalePrice, InternetPricePaid)
           VALUES
           (?, ?, ?, ?, ?) ';
      OPEN ItemCrsr USING SalesInvoiceID, ProductID, ItemQuantity/CaseSize, ItemTotal/ItemQuantity, ItemTotal; 
      CLOSE ItemCrsr;
    END IF;       
  FETCH NEXT FROM Crsr('OrderNumber', 'RecipientName', 'ShippingDetails', 'Email', 'Phone', 
                        'customer_note', 'shipping_total', 'order_subtotal', 'WebKey', 'DateWanted', 
                        'CaseSingle', 'ProductID', 'CaseSize', 'item_total', 'item_quantity')
              INTO OrderNumber, RecipientName, ShippingDetails, Email, PhoneNumber, CustomerNote, ShippingTotal,
                    TotalValue, WebKey, DateWanted, CaseSingle, ProductID, CaseSize, ItemTotal, ItemQuantity;  
  END WHILE;
  
--SET ProcessedOrderNumbers = SUBSTRING(ProcessedOrderNumbers FROM LENGTH(ProcessedOrderNumbers) - 2 FOR 2);   
  
EXECUTE IMMEDIATE
  ' UPDATE WebShopOrders SET "Status" = ''Imported'' ' + 
  ' WHERE UPPER(Status) = ''PROCESSING'' ';

SET Result = COALESCE(Result + #13, '') 
             + COALESCE('WebShop Order Numbers: ' + ProcessedOrderNumbers + #13 + #13, '')  
             + COALESCE(CAST(OrderCount AS VARCHAR), '') 
             + COALESCE(' Sales Invoices added with IDs: ' + SalesInvoiceIDs + #13 + #13, '')  
             + COALESCE('Total Value: ' + CAST(OverallTotalValue AS VARCHAR) + #13, '');
              
CALL EmailRunner('rgurak@equalexchange.coop, jpitts@equalexchange.coop',
                 'EESys Internet Orders imported: ' + CAST(Current_Date AS VARCHAR),
                 'Dear Rachel and Josh, ' + #13 +
                  Result + #13 +                  
                  'These should now be being processed by warehouse staff' + #13 +
                  '',
                  EmailRunnerResult);                 
CLOSE Crsr;
END
Description
'[Properties] 
Type=System
Title=Import Internet Orders
LinkTable=SalesInvoices
UserMessage=Place a CSV file into the "Imports" store folder on the server, and use this in the "aFileName" field.
CheckFirst=true
SecurityLevel=40
ImageIndex=304'
