/*
    EQUAL EXCHANGE INTERNET ORDER AUTOMATION SCRIPT
     * ADD NEW EMAILRUNNER AND PDFRUNNER DLLS TO CONFIG FOLDER * 
    MAY 2023
*/

CREATE MODULE PDFRunner
PATH 'PDFRunner.dll'
!

CREATE MODULE EmailRunner
PATH 'EmailRunner.dll'
!

ALTER PROCEDURE PDFRunner(IN "AppName" VARCHAR,
                           IN "ResourceID" INTEGER, 
                           IN "FileName" VARCHAR,
                           OUT "Result" VARCHAR)
EXTERNAL NAME PDFRunner    
!

ALTER PROCEDURE "EmailRunner" (IN "To" VARCHAR,
                               IN "Subject" VARCHAR,
                               IN "Body" VARCHAR,
                               IN "Attachments" VARCHAR,
                               OUT "Result" VARCHAR)

EXTERNAL Name EmailRunner
!


ALTER PROCEDURE "SaveEmailSettings" (IN "Host" VARCHAR,
                                      IN "Password" VARCHAR,
                                      IN "FromAddress" VARCHAR,
                                      IN "Port" INTEGER,
                                      IN AuthenticationType INTEGER,
                                      OUT "Result" VARCHAR)

EXTERNAL Name EmailRunner
!


CREATE TABLE WebShopOrders
(
  ID INTEGER DEFAULT UID(),
  order_numberVARCHAR(200),
  order_date VARCHAR(200),
  paid_date VARCHAR(200),
  status VARCHAR(200),
  shipping_total VARCHAR(200),
  order_total VARCHAR(200),
  order_subtotal VARCHAR(200),
  order_key VARCHAR(200),
  transaction_id VARCHAR(200),
  billing_email VARCHAR(200),
  billiing_phone VARCHAR(200),
  customer_id VARCHAR(200),
  customer_user VARCHAR(200),
  customer_email VARCHAR(200),
  shipping_first_name VARCHAR(200),
  shipping_last_name VARCHAR(200),
  shipping_company VARCHAR(200),
  shipping_phone VARCHAR(200),
  shipping_address_1 VARCHAR(200),
  shipping_address_2 VARCHAR(200),
  shipping_postcode VARCHAR(200),
  shipping_city VARCHAR(200),
  shipping_state VARCHAR(200),
  shipping_country VARCHAR(200),
  customer_note VARCHAR(200),
  item_name VARCHAR(200),
  item_sku VARCHAR(200),
  item_quantity VARCHAR(200),
  item_subtotal VARCHAR(200),
  item_total VARCHAR(200),
  Complete BOOLEAN DEFAULT false NOT NULL,
  DateCreated TIMESTAMP DEFAULT Current_Timestamp NOT NULL
)
!  

PUBLISH Database "Data"
TABLES "WebShopOrders"
!

CREATE STORE Imports
AS LOCAL PATH 'Stores\Imports'
!

ALTER TABLE SalesInvoiceItems
ADD COLUMN InternetPricePaid DECIMAL(19,4) DEFAULT 0 NOT NULL
!

CREATE FUNCTION ProdIDFromCode(IN aProdCode VARCHAR)
RETURNS INTEGER
BEGIN
  DECLARE Result INTEGER;
  DECLARE Crsr CURSOR FOR Stmt;
SET Result = -1;  
PREPARE Stmt FROM
 ' SELECT ID FROM Products WHERE ProdCode = ? ';
OPEN Crsr USING aProdCode; 
FETCH FIRST FROM Crsr('ID') INTO Result;
CLOSE Crsr;
RETURN Result; 
END
!

INSERT INTO Types 
("Name", LinkTable, LinkField, Description, Color)
 VALUES
('Internet Order', 'CustomerProducts', 'DiscountTypeID', '', 0) 
!

/*
--NOTE: Can't add this PK until the "item_sku" field is corrected.
--ALTER TABLE WebShopOrders
--ADD CONSTRAINT "PK" PRIMARY KEY ("order_number", "item_sku")
--! 

ALTER TABLE WebShopOrders
--ADD COLUMN ID INTEGER DEFAULT UID()
--ADD COLUMN  DateCreated TIMESTAMP DEFAULT Current_Timestamp NOT NULL
--ADD COLUMN BillingPhone VARCHAR(200)


DELETE FROM WebShopOrders
!

IMPORT TABLE WebShopOrders
FROM "Order Export Staging Test.csv"
IN STORE "Imports"
FORMAT DELIMITED
USE HEADERS
!

RENAME FILE "Order Export Staging Test.csv"
IN STORE Imports TO "Order Export Staging Test.csv-old"
!  

SELECT 
  WS.shipping_first_name + ' ' + 
  WS.shipping_last_name as RecipientName, 
  WS.shipping_first_name + ' ' + 
  WS.shipping_last_name + #13 +
  WS.shipping_address_1 + #13 + 
  WS.shipping_address_2 + #13 + 
  WS.shipping_postcode + #13 + 
  WS.shipping_city + #13 + 
  WS.shipping_state + #13 + 
  WS.shipping_country as ShippingDetails,
  WS.billing_email as Email,
  WS.billing_phone as Phone,
  WS.customer_note,
  WS.shipping_total,
  WS.order_subtotal,
  COALESCE(WS.Order_key + ' ', '') + COALESCE(WS.Transaction_id + ' ', '') + COALESCE(WS.Order_Number, '') as WebKeys,
  CAST(WS.order_date as TIMESTAMP DATE FORMAT 'DD/MM/YYYY' TIME FORMAT 'HH:MM') as DateDone,
  WS.*,
  IF(POSITION('-s', WS.item_sku) > 0 THEN 'Single' ELSE 'Case') as Units,
  P.ID,
  P.CountUnits
FROM WebShopOrders WS
LEFT JOIN Products P ON P.ProdCode = WS.item_sku
!

--Review EE.SalesInvoices  SII and add useful additional columns for internet orders.


 --Select a result set which includes and "error" rows with data that does not conform. Email details of this data to staff.
 -- Select Good orders into a result set with LINKs to DB tables (Products)  CASE SIZE  PROD-ID to create source data for the script.
 --Iterate the source-data
   -- Each time the Order_Number changes, insert a SalesInvoice Row with Customer data in appropriate columns.
   -- For every row add a SalesInvoiceItem with the Product, Price and Qty.
   --   This will involve a test for whether the field "Units" = "Single" or "Case"  
   --   For single, SII.Qty = Order_Quantity / CaseSize, for "Case" SII.Qty = Order_Quantity.
   --   Note that in the SII table the DiscountTypeID can be set to TypeID('Internet Order') to demark Internet Orders
   --   Also: All value-computations are irrelevant, as the only paperwork generated is the Delivery Note.
   --   Update the InternetPricePaid with the item_total    
   -- If there is a shipping cost add a row for shipping cost.
   
   
-- ORIXA AUTOMATED SQL FOR Duplicate default CUSTOMERPRODUCTS Type --

SELECT 
  item_sku
 FROM WebShopOrders 
 WHERE NOT item_sku IN
  (SELECT ProdCode FROM Products)
*/  
  
  
  


 

